/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.designer.bpmn.model.process;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import org.ow2.orchestra.designer.bpmn.model.FlowModel;
import org.ow2.orchestra.designer.bpmn.model.IIdentifiableElement;
import org.ow2.orchestra.designer.bpmn.model.data.DataInputModel;
import org.ow2.orchestra.designer.bpmn.model.data.DataMappingModel;
import org.ow2.orchestra.designer.bpmn.model.data.DataObjectModel;

/**
 * @author Loic Albertin
 */
public final class SequenceFlowModel extends FlowModel implements IIdentifiableElement, IBPMNElementWithData {

  /**
   *
   */
  private static final long serialVersionUID = -1159955280613569458L;

  // Data inputs
  private final List<DataInputModel> dataInputs = new ArrayList<DataInputModel>();

  // Data mapping model for send events
  private DataMappingModel dataMappingModel;

  // Output variable
  private List<DataObjectModel> outputVariables = Collections.emptyList();

  private boolean isDefault;
  private boolean hasCondition;

  private String id;

  /**
   * @return the unique id for this element
   */
  public final String getId() {
    return this.id;
  }

  /**
   * @param id the id to set.
   */
  public final void setId(final String id) {
    this.id = id;
  }

  public boolean getIsDefault() {
    return this.isDefault;
  }

  public void setIsDefault(final boolean isDefault) {
    this.isDefault = isDefault;
  }

  public boolean getHasCondition() {
    return hasCondition;
  }

  public void setHasCondition(final boolean hasCondition) {
    this.hasCondition = hasCondition;
  }

  public DataMappingModel getDataMappingModel() {
    return dataMappingModel;
  }

  public void setDataMappingModel(final DataMappingModel dataMappingModel) {
    this.dataMappingModel = dataMappingModel;
  }

  public List<DataInputModel> getDataInputs() {
    return dataInputs;
  }

  public List<DataObjectModel> getOutputVariables() {
    return outputVariables;
  }
}
