/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.designer.models.transformers.bpmn2designer;

import org.ow2.orchestra.designer.bpmn.model.process.GatewayModel;
import org.ow2.orchestra.designer.bpmn.model.process.GatewayType;
import org.ow2.orchestra.designer.models.exceptions.TransformationException;
import org.ow2.orchestra.jaxb.bpmn.TEventBasedGateway;
import org.ow2.orchestra.jaxb.bpmn.TEventBasedGatewayType;
import org.ow2.orchestra.jaxb.bpmn.TExclusiveGateway;
import org.ow2.orchestra.jaxb.bpmn.TGateway;
import org.ow2.orchestra.jaxb.bpmn.TParallelGateway;

/**
 * Transform {@link TGateway}s into {@link GatewayModel}s and vice versa
 *
 * @author Loic Albertin
 */
public final class GatewayTransformer {

  /**
   * Utility classes should not have public or default constructors.
   */
  private GatewayTransformer() {

  }


  /**
   * @param tGateway
   * @param context
   * @return
   * @throws TransformationException
   */
  public static void bpmnToModel(final TGateway tGateway, final Bpmn2DesignerContext context) throws TransformationException {
    final GatewayModel gatewayModel = new GatewayModel();
    gatewayModel.setProcessModel(context.getProcessModel());
    gatewayModel.setName(tGateway.getName());
    if (tGateway instanceof TParallelGateway) {
      gatewayModel.setGatewayType(GatewayType.PARALLEL);
    } else if (tGateway instanceof TExclusiveGateway) {
      gatewayModel.setGatewayType(GatewayType.EXCLUSIVE);
    } else if (tGateway instanceof TEventBasedGateway) {
      if (((TEventBasedGateway) tGateway).getEventGatewayType().equals(TEventBasedGatewayType.EXCLUSIVE)) {
        gatewayModel.setGatewayType(GatewayType.EXCLUSIVE_EVENT);
      }
    }
    context.getProcessModel().getElements().add(gatewayModel);
    context.getModelsMap().put(tGateway, gatewayModel);


    if (tGateway.getId() == null || "".equals(tGateway.getId())) {
      // If there is no id, we will not be able to find out a diagram element
      // for this gateway.
      // We will not be able to reference this gateway from the designer, so this is an error
      throw new TransformationException("BPMN conversion error: A gateway has no id.");
    }
    gatewayModel.setId(tGateway.getId());
    // Now retrieve positions
    ShapeTransformer.bpmnToModel(tGateway.getId(), gatewayModel, context);
  }

}
