/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.designer.models.transformers.designer2bpmn;

import javax.xml.XMLConstants;
import javax.xml.namespace.QName;
import org.ow2.orchestra.designer.bpmn.model.data.type.XSDSimpleType;
import org.ow2.orchestra.designer.bpmn.model.operations.CorrelationMappingModel;
import org.ow2.orchestra.designer.bpmn.model.operations.CorrelationModel;
import org.ow2.orchestra.designer.models.exceptions.TransformationException;
import org.ow2.orchestra.jaxb.bpmn.CorrelationKey;
import org.ow2.orchestra.jaxb.bpmn.CorrelationPropertyRetrievalExpression;
import org.ow2.orchestra.jaxb.bpmn.MessageFlow;
import org.ow2.orchestra.jaxb.bpmn.TConversation;
import org.ow2.orchestra.jaxb.bpmn.TConversationNode;
import org.ow2.orchestra.jaxb.bpmn.TCorrelationProperty;
import org.ow2.orchestra.jaxb.bpmn.TFormalExpression;
import org.ow2.orchestra.jaxb.bpmn.TMessage;
import org.ow2.orchestra.util.Misc;

/**
 * Transform {@link org.ow2.orchestra.designer.bpmn.model.operations.CorrelationModel}s into {@link org.ow2.orchestra.jaxb.bpmn.TCorrelationProperty}
 *
 * @author Guillaume Porcher
 */
public final class CorrelationTransformer {

  /**
   * Utility classes should not have public or default constructors.
   */
  private CorrelationTransformer() {
  }

  public static TCorrelationProperty correlationToBPMN(final CorrelationModel correlationModel, final Designer2BpmnContext context) throws TransformationException {
    if (!context.getCorrelationsMap().containsKey(correlationModel)) {
      TCorrelationProperty correlationProperty = new TCorrelationProperty();
      correlationProperty.setName(correlationModel.getName());
      correlationProperty.setId(correlationModel.getId());
      if (correlationModel.getType() == null) {
        // TODO: remove this
        XSDSimpleType simpleType = new XSDSimpleType();
        simpleType.setQName(QName.valueOf("{" + XMLConstants.W3C_XML_SCHEMA_NS_URI + "}string"));
        simpleType.setId(Misc.getUniqueId("_"));
        correlationModel.setType(simpleType);
      }
      ElementTransformer.modelToBpmnItemDefinition(correlationModel.getType(), context);
      correlationProperty.setType(new QName(
              context.getDefinitions().getTargetNamespace(),
              correlationModel.getType().getId()));
      context.getDefinitions().getRootElements().add(context.getBpmnObjectFactory().createCorrelationProperty(correlationProperty));
      context.getCorrelationsMap().put(correlationModel, correlationProperty);
    }
    return context.getCorrelationsMap().get(correlationModel);
  }

  public static TConversationNode correlationToBPMNConversation(final CorrelationModel correlationModel, final Designer2BpmnContext context) throws TransformationException {
    if (!context.getConversationsMap().containsKey(correlationModel)) {
      TConversation conversation = new TConversation();
      conversation.setName(correlationModel.getName());
      conversation.setId("conversation" + correlationModel.getId());
      CorrelationKey correlationKey = new CorrelationKey();
      correlationKey.setId("ck" + correlationModel.getId());
      correlationKey.getCorrelationPropertyReves().add(new QName(context.getDefinitions().getTargetNamespace(), correlationModel.getId()));
      conversation.getCorrelationKeies().add(correlationKey);
      context.getCollaboration().getConversationNodes().add(context.getBpmnObjectFactory().createConversation(conversation));
      context.getConversationsMap().put(correlationModel, conversation);
    }
    return context.getConversationsMap().get(correlationModel);
  }

  public static void correlationMappingToBPMN(final CorrelationMappingModel correlationMappingModel, final MessageFlow messageFlow, final Designer2BpmnContext context) throws TransformationException {
    // generate correlation property retrieval expression
    TCorrelationProperty correlationProperty = correlationToBPMN(correlationMappingModel.getCorrelationModel(), context);
    CorrelationPropertyRetrievalExpression correlationPropertyRetrievalExpression = new CorrelationPropertyRetrievalExpression();
    TMessage message = Designer2BpmnTransformer.generateMessage(correlationMappingModel.getMessageModel(), context);
    correlationPropertyRetrievalExpression.setMessageRef(new QName(context.getDefinitions().getTargetNamespace(), message.getId()));
    TFormalExpression messagePath = new TFormalExpression();
    if (correlationMappingModel.getChildNodeSelector() != null) {
      QName childNodeQName = QName.valueOf(correlationMappingModel.getChildNodeSelector());
      String prefix = Misc.getHumanReadableId("ns");
      messagePath.getOtherAttributes().put(new QName(XMLConstants.XMLNS_ATTRIBUTE_NS_URI, prefix), childNodeQName.getNamespaceURI());
      messagePath.getContent().add("/" + correlationMappingModel.getMessagePartModel().getPartName() + "/" + prefix + ":" + childNodeQName.getLocalPart());
    } else {
      messagePath.getContent().add("/" + correlationMappingModel.getMessagePartModel().getPartName());
    }
    correlationPropertyRetrievalExpression.setMessagePath(messagePath);
    correlationProperty.getCorrelationPropertyRetrievalExpressions().add(correlationPropertyRetrievalExpression);

    // add messageFlow to conversation
    TConversationNode conversationNode = correlationToBPMNConversation(correlationMappingModel.getCorrelationModel(), context);
    conversationNode.getMessageFlowReves().add(new QName(context.getDefinitions().getTargetNamespace(), messageFlow.getId()));
  }
}
