/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.designer.models.transformers.designer2bpmn;

import java.util.HashMap;
import java.util.Map;
import javax.xml.bind.JAXBElement;
import javax.xml.namespace.QName;
import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
import org.ow2.orchestra.designer.bpmn.model.data.type.DataTypeModel;
import org.ow2.orchestra.designer.bpmn.model.data.type.WSDLMessageModel;
import org.ow2.orchestra.designer.bpmn.model.operations.CorrelationModel;
import org.ow2.orchestra.jaxb.bpmn.Definitions;
import org.ow2.orchestra.jaxb.bpmn.TCollaboration;
import org.ow2.orchestra.jaxb.bpmn.TConversationNode;
import org.ow2.orchestra.jaxb.bpmn.TCorrelationProperty;
import org.ow2.orchestra.jaxb.bpmn.TFlowElement;
import org.ow2.orchestra.jaxb.bpmn.TItemDefinition;
import org.ow2.orchestra.jaxb.bpmn.TMessage;
import org.ow2.orchestra.jaxb.bpmn.TProcess;
import org.ow2.orchestra.jaxb.bpmn.bpmndi.BPMNEdge;
import org.ow2.orchestra.jaxb.bpmn.bpmndi.BPMNPlane;
import org.ow2.orchestra.jaxb.bpmn.bpmndi.BPMNShape;
import org.ow2.orchestra.jaxb.bpmn.di.DiagramElement;

/**
 * Class holding the context of a transformation from a Designer model into a
 * BPMN model.
 *
 * @author Loic Albertin
 */
public final class Designer2BpmnContext {

  /**
   * Objects factory for {@link JAXBElement}s within the
   * org.ow2.orchestra.jaxb.bpmn package
   */
  private final org.ow2.orchestra.jaxb.bpmn.ObjectFactory bpmnObjectFactory = new org.ow2.orchestra.jaxb.bpmn.ObjectFactory();
  /**
   * Objects factory for {@link JAXBElement}s within the
   * org.ow2.orchestra.jaxb.bpmn.bpmndi package
   */
  private final org.ow2.orchestra.jaxb.bpmn.bpmndi.ObjectFactory bpmndiObjectFactory =
    new org.ow2.orchestra.jaxb.bpmn.bpmndi.ObjectFactory();

  /**
   * Objects factory for {@link JAXBElement}s within the
   * org.ow2.orchestra.jaxb.bpmn.dc package
   */
  private final org.ow2.orchestra.jaxb.bpmn.dc.ObjectFactory bpmnDcObjectFactory = new org.ow2.orchestra.jaxb.bpmn.dc.ObjectFactory();

  /**
   * The original {@link Definitions} model
   */
  private Definitions definitions = null;

  /**
   * The original {@link TProcess}
   */
  private TProcess tProcess = null;

  /**
   * The original {@link TCollaboration}
   */
  private TCollaboration collaboration = null;

  /**
   * The original {@link BPMNPlane}
   */
  private BPMNPlane bpmnPlane = null;

  /**
   * A {@link Map} between {@link AbstractElement} and the corresponding
   * {@link TFlowElement}. This map is populated when transforming a
   * {@link AbstractElement} into an {@link TFlowElement}
   */
  private final Map<AbstractElement, TFlowElement> modelsMap = new HashMap<AbstractElement, TFlowElement>();

  /**
   * A {@link Map} between {@link DataTypeModel} and the corresponding
   * {@link TItemDefinition}. This map is populated when transforming a
   * {@link DataTypeModel} into an {@link TItemDefinition}
   */
  private final Map<DataTypeModel, TItemDefinition> dataTypeMap = new HashMap<DataTypeModel, TItemDefinition>();

  /**
   * A {@link Map} between {@link CorrelationModel} and the corresponding
   * {@link TCorrelationProperty}. This map is populated when transforming a
   * {@link CorrelationModel} into an {@link TCorrelationProperty}
   */
  private final Map<CorrelationModel, TCorrelationProperty> correlationsMap = new HashMap<CorrelationModel, TCorrelationProperty>();
  private final Map<CorrelationModel, TConversationNode> conversationsMap = new HashMap<CorrelationModel, TConversationNode>();

  private final Map<WSDLMessageModel, TMessage> messagesMap = new HashMap<WSDLMessageModel, TMessage>();
  /**
   * @return the bpmnObjectFactory
   */
  public org.ow2.orchestra.jaxb.bpmn.ObjectFactory getBpmnObjectFactory() {
    return this.bpmnObjectFactory;
  }

  /**
   * @return the bpmndiObjectFactory
   */
  public org.ow2.orchestra.jaxb.bpmn.bpmndi.ObjectFactory getBpmndiObjectFactory() {
    return this.bpmndiObjectFactory;
  }

  /**
   * @return the bpmnDcObjectFactory
   */
  public org.ow2.orchestra.jaxb.bpmn.dc.ObjectFactory getBpmnDcObjectFactory() {
    return this.bpmnDcObjectFactory;
  }

  /**
   * @return the definitions
   */
  public Definitions getDefinitions() {
    return this.definitions;
  }

  /**
   * @param definitions the definitions to set
   */
  public void setDefinitions(final Definitions definitions) {
    this.definitions = definitions;
  }

  /**
   * @return the tProcess
   */
  public TProcess getTProcess() {
    return this.tProcess;
  }

  /**
   * @param tProcess the tProcess to set
   */
  public void setTProcess(final TProcess tProcess) {
    this.tProcess = tProcess;
  }

  /**
   * @return the modelsMap
   */
  public Map<AbstractElement, TFlowElement> getModelsMap() {
    return this.modelsMap;
  }

  public Map<DataTypeModel, TItemDefinition> getDataTypeMap() {
    return dataTypeMap;
  }

  /**
   * @return the bpmnPlane
   */
  public BPMNPlane getBpmnPlane() {
    return this.bpmnPlane;
  }

  /**
   * @param bpmnPlane the bpmnPlane to set
   */
  public void setBpmnPlane(final BPMNPlane bpmnPlane) {
    this.bpmnPlane = bpmnPlane;
  }

  /**
   * This method iterates over {@link DiagramElement}s contained into context's
   * {@link BPMNPlane} to find out a {@link BPMNPlane}, {@link BPMNShape} or
   * {@link BPMNEdge} having a BPMNElement attribute equals to name.
   *
   * @param name The {@link QName} representation of the BPMNElement to find.
   * @return Returns the first matching element or <code>null</code> if none
   *         match.
   * @see BPMNPlane
   * @see BPMNShape
   * @see BPMNEdge
   */
  public DiagramElement getDiagramElementOnPlane(final QName name) {
    if (name == null || this.bpmnPlane == null) {
      return null;
    }
    for (final JAXBElement<? extends DiagramElement> jaxbDiag : this.bpmnPlane.getDiagramElements()) {
      final DiagramElement element = jaxbDiag.getValue();
      if (element instanceof BPMNPlane && name.equals(((BPMNPlane) element).getBpmnElement())) {
        return element;
      } else if (element instanceof BPMNShape && name.equals(((BPMNShape) element).getBpmnElement())) {
        return element;
      } else if (element instanceof BPMNEdge && name.equals(((BPMNEdge) element).getBpmnElement())) {
        return element;
      }
    }
    return null;
  }

  public void setCollaboration(final TCollaboration tCollaboration) {
    this.collaboration = tCollaboration;
  }

  public TCollaboration getCollaboration() {
    return this.collaboration;
  }

  public Map<CorrelationModel, TCorrelationProperty> getCorrelationsMap() {
    return correlationsMap;
  }

  public Map<WSDLMessageModel, TMessage> getMessagesMap() {
    return messagesMap;
  }

  public Map<CorrelationModel, TConversationNode> getConversationsMap() {
    return conversationsMap;
  }
}
